package com.link.core.cache;

import java.lang.reflect.Method;
import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.aop.support.AopUtils;

public class CacheInterceptor implements MethodInterceptor {
   Logger logger = LoggerFactory.getLogger(CacheInterceptor.class);
   private final CacheHandler cacheHandler;
   private final CacheConfigure cacheConfigure;

   public CacheInterceptor(CacheHandler cacheHandler, CacheConfigure cacheConfigure) {
      this.cacheHandler = cacheHandler;
      this.cacheConfigure = cacheConfigure;
   }

   public Object invoke(MethodInvocation methodInvocation) throws Throwable {
      if (!this.cacheConfigure.isEnable()) {
         return methodInvocation.proceed();
      } else {
         Class<?> cls = AopProxyUtils.ultimateTargetClass(methodInvocation.getThis());
         if (cls == null && cls != null) {
            cls = methodInvocation.getThis().getClass();
         }

         if (!cls.equals(methodInvocation.getThis().getClass())) {
            if (this.logger.isDebugEnabled()) {
               this.logger.debug(methodInvocation.getThis().getClass() + "-->" + cls);
            }

            return methodInvocation.proceed();
         } else {
            if (this.logger.isDebugEnabled()) {
               this.logger.debug(methodInvocation.toString());
            }

            Method method = methodInvocation.getMethod();
            if (method.isAnnotationPresent(LinkCache.class)) {
               LinkCache cache = (LinkCache)method.getAnnotation(LinkCache.class);
               if (this.logger.isDebugEnabled()) {
                  this.logger.debug(methodInvocation.getThis().getClass().getName() + "." + method.getName() + "-->@Cache");
               }

               return this.cacheHandler.proceed(new CacheAopProxy(methodInvocation), cache);
            } else {
               Method specificMethod = AopUtils.getMostSpecificMethod(method, methodInvocation.getThis().getClass());
               if (specificMethod.isAnnotationPresent(LinkCache.class)) {
                  LinkCache cache = (LinkCache)specificMethod.getAnnotation(LinkCache.class);
                  if (this.logger.isDebugEnabled()) {
                     this.logger.debug(methodInvocation.getThis().getClass().getName() + "." + specificMethod.getName() + "-->@Cache");
                  }

                  return this.cacheHandler.proceed(new CacheAopProxy(methodInvocation), cache);
               } else {
                  return methodInvocation.proceed();
               }
            }
         }
      }
   }
}
